@@ -82,7 +82,7 @@ gchar*
 gnome_vfs_format_file_size_for_display (GnomeVFSFileSize size)
 {
 	if (size < (GnomeVFSFileSize) KILOBYTE_FACTOR) {
-		return g_strdup_printf (dngettext(GETTEXT_PACKAGE, "%u byte", "%u bytes",(guint) size), (guint) size);
+		return g_strdup_printf (ngettext("%u byte", "%u bytes",(guint) size), (guint) size);
 	} else {
 		gdouble displayed_size;
 
@@ -835,12 +835,7 @@ gnome_vfs_get_volume_free_space (const GnomeVFSURI *vfs_uri,
 		{
 			return GNOME_VFS_ERROR_NOT_SUPPORTED;
 		}
-	} else {
-		/* everything is copacetic... free the unescaped path */
-		g_free (unescaped_path);
 	}
-#else 
-	g_free (unescaped_path);
 #endif
 
 	block_size = statfs_buffer.f_bsize; 
@@ -1754,8 +1749,8 @@ gnome_vfs_get_uri_scheme (const char *uri)
 	if (colon == NULL) {
 		return NULL;
 	}
-
-	return g_ascii_strdown (uri, colon - uri);
+	
+	return g_strndup (uri, colon - uri);
 }
 
 /* Note that NULL's and full paths are also handled by this function.
