@@ -169,7 +169,7 @@ static GnomeVFSResult read_response_line(NNTPConnection *conn, char **line) {
 		/* we don't have a full line. Lets read some... */
 		bytes_read = 0;
 		result = gnome_vfs_socket_buffer_read (conn->socketbuf, buf,
-						       bytes, &bytes_read);
+						       bytes, &bytes_read, NULL);
 		buf[bytes_read] = '\0';
 		conn->response_buffer = g_string_append (conn->response_buffer,
 							 buf);
@@ -243,8 +243,9 @@ static GnomeVFSResult do_control_write (NNTPConnection *conn,
         char *actual_command = g_strdup_printf ("%s\r\n", command);
 	GnomeVFSFileSize bytes = strlen (actual_command), bytes_written;
 	GnomeVFSResult result = gnome_vfs_socket_buffer_write (conn->socketbuf,
-							       actual_command, bytes, &bytes_written);
-	gnome_vfs_socket_buffer_flush (conn->socketbuf);
+							       actual_command, bytes, &bytes_written,
+							       NULL);
+	gnome_vfs_socket_buffer_flush (conn->socketbuf, NULL);
 	g_free (actual_command);
 
 	return result;
@@ -369,7 +370,7 @@ nntp_connection_create (NNTPConnection **connptr, GnomeVFSURI *uri, GnomeVFSCont
 			/* login failed */
 			g_warning ("NNTP server said: \"%d %s\"\n", conn->response_code,
 			   	conn->response_message);
-			gnome_vfs_socket_buffer_destroy (conn->socketbuf, FALSE);
+			gnome_vfs_socket_buffer_destroy (conn->socketbuf, FALSE, context ? gnome_vfs_context_get_cancellation(context) : NULL);
 			gnome_vfs_inet_connection_destroy (conn->inet_connection, NULL);
 			g_free (conn);
 			return result;
@@ -395,7 +396,7 @@ nntp_connection_destroy (NNTPConnection *conn)
 	}
 	
 	if (conn->socketbuf) 
-	        gnome_vfs_socket_buffer_destroy (conn->socketbuf, FALSE);
+	        gnome_vfs_socket_buffer_destroy (conn->socketbuf, FALSE, NULL);
 
 	gnome_vfs_uri_unref (conn->uri);
 
